# DO NOT EDIT! This file is automatically generated
import typing

from commercetools._schemas._graph_ql import GraphQLResponseSchema
from commercetools.helpers import RemoveEmptyValuesMixin
from commercetools.types._graph_ql import GraphQLResponse
from commercetools.typing import OptionalListStr

from . import abstract, traits


class GraphqlService(abstract.AbstractService):
    """The commercetools™ platform provides a GraphQL API"""

    def create(self) -> GraphQLResponse:
        """Execute a GraphQL query

        The commercetools™ platform provides a GraphQL API
        """
        params: typing.Dict[str, str] = {}
        return self._client._post(
            endpoint="graphql",
            params=params,
            data_object=None,
            request_schema_cls=None,
            response_schema_cls=GraphQLResponseSchema,
        )
